/** 
 * Copyright (C) 2009 "Darwin V. Felix" <dfelix@users.sourceforge.net>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

import java.util.Iterator;

import javax.security.auth.login.LoginException;
import javax.xml.namespace.QName;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;

import net.sourceforge.spnego.SpnegoSOAPConnection;

public class ExampleSpnegoSOAPClient {
    
    private final String module;
    private final String kuser;
    private final String kpass;
    private String serviceLocation = null;
    private String namespace = null;
    private String methodName = null;
    private String nsprefix = null;
    
    public ExampleSpnegoSOAPClient(final String loginModuleName,
        final String username, final String password) throws LoginException {
        
        // assert
        assertNotEmpty(loginModuleName);
        assertNotEmpty(username);
        assertNotEmpty(password);
        
        this.module = loginModuleName;
        this.kuser = username;
        this.kpass = password;
    }

    public void setServiceLocation(final String url) {
        
        // assert
        assertNotEmpty(url);
        
        this.serviceLocation = url;
    }
    
    public void setMethodName(final String namespaceURI, final String localPart
        , final String prefix) {

        // assert
        assertNotEmpty(namespaceURI);
        assertNotEmpty(localPart);
        assertNotEmpty(prefix);
        
        this.namespace = namespaceURI;
        this.methodName = localPart;
        this.nsprefix = prefix;
    }
    
    public SOAPElement invoke(final String... args) throws LoginException
        , SOAPException {
        
        SOAPMessage response = null;
        final SpnegoSOAPConnection conn = 
            new SpnegoSOAPConnection(this.module, this.kuser, this.kpass);
        
        try {
            final MessageFactory msgFactory = MessageFactory.newInstance();
            final SOAPMessage message = msgFactory.createMessage();

            final SOAPBody body = message.getSOAPBody();
            
            final SOAPBodyElement bodyElement = body.addBodyElement(
                    new QName(this.namespace, this.methodName, this.nsprefix));
            
            for (int i=0; i<args.length; i++) {
                final SOAPElement element = bodyElement.addChildElement(
                        new QName("arg" + i));

                element.addTextNode(args[i]);
            }
            
            response = conn.call(message, this.serviceLocation);

        } finally {
            conn.close();
        }
        
        SOAPElement soapElement;
        final Iterator<?> iter = response.getSOAPBody().getChildElements();
        
        if (iter.hasNext()) {
            final Object obj = iter.next();
            if (obj instanceof SOAPElement) {
                soapElement = (SOAPElement) obj;
            } else {
                throw new SOAPException("Not a SOAPElement: " + obj.toString());
            }
        } else {
            throw new SOAPException("Iterator has no elements.");
        }
        
        return soapElement;
    }
    
    private void assertNotEmpty(final String param) {
        if (null == param || param.length() == 0) {
            throw new IllegalArgumentException("param value must not be empty.");
        }
    }
}
